import React, { PureComponent } from 'react';
import { Button, Icon, Table, Alert } from 'antd';
import Logger from '../../utils/logger';
import styles from './index.less';

const logger = Logger.getLogger('StandardTableComponent');

class StandardTable extends PureComponent {
  state = {
    selectedRowKeys: [],
    totalCallNo: 0,
  }

  componentWillReceiveProps(nextProps) {
    // clean state
    if (nextProps.selectedRows.length === 0) {
      this.setState({
        selectedRowKeys: [],
        totalCallNo: 0,
      });
    }
  }

  handleRowSelectChange = (selectedRowKeys, selectedRows) => {
    const totalCallNo = selectedRows.reduce((sum, val) => {
      return sum + parseFloat(val.callNo, 10);
    }, 0);

    if (this.props.onSelectRow) {
      this.props.onSelectRow(selectedRows);
    }

    this.setState({ selectedRowKeys, totalCallNo });
  }

  handleTableChange = (pagination, filters, sorter) => {
    this.props.onChange(pagination, filters, sorter);
  }

  cleanSelectedKeys = () => {
    this.handleRowSelectChange([], []);
  }

  render() {
    const { selectedRowKeys, totalCallNo } = this.state;
    const { dataSource, pagination, columns, rowKeyName, loading } = this.props;

    //配置分页器
    const paginationProps = {
      showSizeChanger: true,
      showQuickJumper: true,
      ...pagination,
    };

    //确定该行是否可选，object
    const rowSelection = {
      selectedRowKeys, //指定选中项的key数组，需要和onChange配合使用，string[]
      onChange: this.handleRowSelectChange, //选中项发生变化时的回调
      getCheckboxProps: record => ({ //选择框的默认属性配置，func，这里设置为所有均可选
        // disabled: record.disabled,
        disabled: false,
      }),
    };

    return (
      <div className={styles.standardTable}>
        <div className={styles.tableAlert}>
          <Alert
            message={(
              <p>
                已选择 <a style={{ fontWeight: 600 }}>{selectedRowKeys.length}</a> 项
                <a onClick={this.cleanSelectedKeys} style={{ marginLeft: 12 }}>清空</a>
              </p>
            )}
            type="info"
            showIcon
          />
        </div>
        <Table
          loading={loading}
          rowKey={rowKeyName}
          rowSelection={rowSelection}
          dataSource={dataSource}
          columns={columns}
          pagination={paginationProps}
          onChange={this.handleTableChange}
        />
      </div>
    );
  }
}

export default StandardTable;
