import { getResourceList, getOssUploadParams, uploadToOss, submitOssUploadResult } from '../services/resource';
import { Notification } from 'antd';

export default {
  namespace: 'resource',

  state: {
    oss: {
      fileList: [],
      ossParams: {},
    },
    data: {
      list: [],
      pagination: {},
    },
    loading: true,
    ossUploading: false,
  },

  effects: {
    *fetch({ payload }, { put, call }) {
      yield put({
        type: 'changeLoading',
        payload: {
          loading: true
        },
      });

      console.log('=====================');
      console.log(payload);
      console.log('=====================');
      const response = yield call(getResourceList, payload);
      const data = {
        list: response.data.list || [],
        pagination: {
          pageSize: response.data.pageSize || 10,
          current: response.data.pageNo || 1,
          total: response.data.totalSize || 0,
        }
      };

      yield put({
        type: 'save',
        payload: data,
      });

      yield put({
        type: 'changeLoading',
        payload: {
          loading: false,
        }
      });
    },

    *resourceUpload({ payload, callback }, { put, call, select }) {
      //Step-1 初始化上传状态为true
      yield put({ type: 'ossUploading', payload: { ossUploading: true} });

      //Step-2 进行oss上传参数有效期校验
      let ossUploadArgs = yield select(state => state.resource.ossParams);
      let cacheTimestamp = (Date.parse(new Date()) / 1000 + 5).toString();//加5s缓冲时间
      if (!ossUploadArgs || !ossUploadArgs.expire || ossUploadArgs.expire <= cacheTimestamp) {
        const ossArgsUpdateRes = yield call(getOssUploadParams);
        if (!ossArgsUpdateRes.code || ossArgsUpdateRes.code !== 200) {
          yield put({ type: 'ossUploading', payload: { ossUploading: false, fileList: [] } });
          Notification.error({ message: '图片上传失败!' });
          return;
        }
        yield put({ type: 'ossParamsSave', payload: ossArgsUpdateRes.data });
      }

      //Step-3 oss文件上传
      const ossParams = yield select(state => state.resource.ossParams);
      const fileList = yield select(state => state.resource.fileList);
      const ossUploadRes = yield call(uploadToOss, {fileList, ossParams});

      //Step-4 oss上传成功后，提交信息到后台
      if (!ossUploadRes.code || ossUploadRes.code !== 200) {
        yield put({ type: 'ossUploading', payload: { ossUploading: false, fileList: [] } });
        Notification.error({ message: '图片上传失败!' });
        return;
      }
      const submitRes  = yield call(submitOssUploadResult, ossUploadRes.data);
      (!submitRes.code || submitRes.code !== 200 || !submitRes.success) ?
        Notification.error({ message: '图片上传失败!' }) :
        Notification.success({ message: '图片上传成功!' });
      yield put({ type: 'ossUploading', payload: { ossUploading: false, fileList: [] } });
    }
  },
  reducers: {
    /* 保存请求回来的数据 */
    save(state, action) {
      return {
        ...state,
        data: action.payload,
      };
    },
    /* 控制列表的加载状态 */
    changeLoading(state, action) {
      return {
        ...state,
        ...action.payload,
      };
    },
    /* oss参数的保存，刷新页面后失效 */
    ossParamsSave(state, action) {
      return {
        ...state,
        ossParams: action.payload,
      }
    },
    /* 改变文件的上传状态，清空上传文件列表 */
    ossUploading(state, action) {
      return {
        ...state,
        ...action.payload,
      }
    },
    /* 控制fileList内容 */
    setFileList(state, action) {
      return {
        ...state,
        ...action.payload,
      }
    }
  }
}
