import pathToRegexp from 'path-to-regexp';
import { notification, message } from 'antd';
import { getItemList, getItemItem, addItemItem, updateItemItem, delItemItem } from '../services/ItemAPI';
import { dateFormat } from '../utils/utils';
import Logger from '../utils/logger';

const logger = new Logger('ItemModel');

export default {
  namespace: 'item',

  state: {
    data: {
      list: [],
      pagination: {},
    },
    loading: false,
    itemItem: {},
  },

  effects: {
    *getItemList({ payload }, { put, call }) {
      yield put({ type: 'changeLoading', payload: { loading: true } });

      const response = yield call(getItemList, payload);

      const data = {
        list: dateFormat(response.data.list || []),
        pagination: {
          pageSize : response.data.pageSize || 15,
          current  : response.data.pageNo || 1,
          total    : response.data.totalSize || 0,
        }
      };

      yield put({ type: 'saveItemList', payload: { data } });
      yield put({ type: 'changeLoading', payload: { loading: false }});
    },
    *getItemOne({ payload }, { put, call }) {
      const response = yield call(getItemItem, payload);

      const data = response.data || {};

      yield put({ type: 'saveItemData', payload: data });
    },
    *addItemItem({ payload }, { put, call }) {
      const response = yield call(addItemItem, payload);
      if (!response || response.code !== 200) {
        notification.error({
          message: '创建商品失败!',
          description: response.message,
        });
      }
      else {
        notification.success({
          message: '创商品成功!',
          description: '成功创建1个商品',
        });
      }
    },
    *updateItemItem({ payload }, { put, call }) {
      const response = yield call(updateItemItem, payload);
      if (!response || response.code !== 200) {
        notification.error({
          message: '更新商品失败!',
          description: response.message,
        });
      }
      else {
        notification.success({
          message: '更新商品成功!',
          description: '成功更新1个商品',
        });
      }
    },
    *delItemItem({ payload }, { put, call }) {
      const response = yield call(delItemItem, payload);
      if (!response || response.code !== 200) {
        notification.error({
          message: '删除商品失败!',
          description: response.message,
        });
      }
      else {
        notification.success({
          message: '删除商品成功!',
          description: '成功删除1个商品',
        });
      }
    }
  },

  reducers: {
    saveItemList(state, action) {
      return {
        ...state,
        ...action.payload,
      }
    },
    changeLoading(state, action) {
      return {
        ...state,
        ...action.payload,
      }
    },
    saveItemData(state, action) {
      return {
        ...state,
        itemItem: { ...action.payload },
      }
    },
    clearAll(state) {
      return {
        ...state,
        itemItem: {},
      }
    }
  },
}
