import { getMerchantList } from '../services/MerchantApi';
import Logger from '../utils/logger';
import { dateFormat } from '../utils/utils';

const logger = new Logger('MerchantModel');

export default {
  namespace: 'merchant',

  state: {
    merchantList: [],
  },

  effects: {
    *getAllMerchants({ payload }, { put, call }) {
      const response = yield call(getMerchantList);

      if (response.err) {
        logger.error('getAllMerchants request error: %o', response.err);
        return;
      }
      else {
        logger.debug('getAllMerchants request success: %o', response);
      }

      yield put({
        type: 'saveMerchantList',
        payload: dateFormat(response.data.list || []),
      });
    },
  },

  reducers: {
    saveMerchantList(state, action) {
      return {
        ...state,
        merchantList: action.payload,
      }
    },
  }
}
