import pathToRegexp from 'path-to-regexp';
import { notification, message } from 'antd';
import { getSupportList, getSupportItem, addSupportItem, updateSupportItem, delSupportItem } from '../services/ProductApi';
import { dateFormat } from '../utils/utils';
import Logger from '../utils/logger';

const logger = new Logger('SupportModel');

function SupportItemFormTemp() {
  return {
    id: '',
    code: '',
    name: '',
    digest: '',
    status: '',
    cpId: '',
    cpName: '',
    tags: [],
    supportArr: [],
  };
}

export default {
  namespace: 'support',

  state: {
    data: {
      list: [],
      pagination: {},
    },
    loading: false,
    supportItem: SupportItemFormTemp(),
  },

  effects: {
    *getSupportList({ payload }, { put, call }) {
      yield put({ type: 'changeLoading', payload: { loading: true } });

      const response = yield call(getSupportList, payload);

      const data = {
        list: dateFormat(response.data.list || []),
        pagination: {
          pageSize : response.data.pageSize || 15,
          current  : response.data.pageNo || 1,
          total    : response.data.totalSize || 0,
        }
      };

      yield put({ type: 'saveSupportList', payload: { data } });
      yield put({ type: 'changeLoading', payload: { loading: false }});
    },
    *getSupportOne({ payload }, { put, call }) {
      const response = yield call(getSupportItem, payload);

      const data = response.data || {};

      yield put({ type: 'saveItemData', payload: data });
    },
    *addSupportItem({ payload }, { put, call }) {
      const response = yield call(addSupportItem, payload);
      if (!response || response.code !== 200) {
        notification.error({
          message: '创建周边失败!',
          description: response.message,
        });
      }
      else {
        notification.success({
          message: '创周边成功!',
          description: '成功创建1个周边',
        });
      }
    },
    *updateSupportItem({ payload }, { put, call }) {
      const response = yield call(updateSupportItem, payload);
      if (!response || response.code !== 200) {
        notification.error({
          message: '更新周边失败!',
          description: response.message,
        });
      }
      else {
        notification.success({
          message: '更新周边成功!',
          description: '成功更新1个周边',
        });
      }
    },
    *delSupportItem({ payload }, { put, call }) {
      const response = yield call(delSupportItem, payload);
      if (!response || response.code !== 200) {
        notification.error({
          message: '删除周边失败!',
          description: response.message,
        });
      }
      else {
        notification.success({
          message: '删除周边成功!',
          description: '成功删除1个周边',
        });
      }
    }
  },

  reducers: {
    saveSupportList(state, action) {
      return {
        ...state,
        ...action.payload,
      }
    },
    changeLoading(state, action) {
      return {
        ...state,
        ...action.payload,
      }
    },
    saveItemData(state, action) {
      return {
        ...state,
        supportItem: {
          id: action.payload.id,
          code: action.payload.code,
          name: action.payload.name,
          digest: action.payload.digest,
          status: action.payload.status,
          cpId: action.payload.cpId,
          cpName: action.payload.cpName,
          tags: action.payload.tags,
          supportArr: action.payload.supportArr,
        }
      }
    },
    clearAll(state) {
      return {
        ...state,
        supportItem: SupportItemFormTemp(),
      }
    }
  },
}
