import { notification } from 'antd';
import { getTagList, getTagItem, addTagItem, updateTagItem, delTagItem } from '../services/ProductApi';
import Logger from '../utils/logger';
import { dateFormat } from '../utils/utils';

const logger = new Logger('TagModel');

function TagItemFormTemp() {
  return {
    tagId: '',
    tagName: '',
    type: '',
    merchantId: '',
    merchantName: '',
  };
}

export default {
  namespace: 'tag',

  state: {
    data: {
      list: [],
      pagination: {},
    },
    loading: false,
    tagItem: TagItemFormTemp(),
  },

  effects: {
    *getTagList({ payload }, { put, call }) {
      yield put({ type: 'changeLoading', payload: { loading: true } });
      const response = yield call(getTagList, payload);

      if (response.err)
      {
        logger.error('Tag list request error: %o', response.err);
        return;
      }
      else {
        logger.debug('Tag list request succ, response: %o', response);
      }

      const data = {
        list: dateFormat(response.data.list || []),
        pagination: {
          pageSize : response.data.pageSize || 15,
          current  : response.data.pageNo || 1,
          total    : response.data.totalSize || 0,
        }
      };
      yield put({ type: 'saveTagList', payload: { data } });
      yield put({ type: 'changeLoading', payload: { loading: false }});
    },
    *getTagOne({ payload }, { put, call }) {
      const response = yield call(getTagItem, payload);

      const data = response.data || {};

      yield put({ type: 'saveItemData', payload: data });
    },
    *addTagItem({ payload }, { put, call }) {
      const response = yield call(addTagItem, payload);
      if (!response || response.code !== 200) {
        notification.error({
          message: '创建标签失败!',
          description: response.message,
        });
      }
      else {
        notification.success({
          message: '创标签成功!',
          description: '成功标签1个周边',
        });
      }
    },
    *updateTagItem({ payload }, { put, call }) {
      const response = yield call(updateTagItem, payload);
      if (!response || response.code !== 200) {
        notification.error({
          message: '更新标签失败!',
          description: response.message,
        });
      }
      else {
        notification.success({
          message: '更新标签成功!',
          description: '成功更新1个标签',
        });
      }
    },
    *delTagItem({ payload }, { put, call }) {
      const response = yield call(delTagItem, payload);
      if (!response || response.code !== 200) {
        notification.error({
          message: '删除标签失败!',
          description: response.message,
        });
      }
      else {
        notification.success({
          message: '删除标签成功!',
          description: '成功删除1个标签',
        });
      }
    }
  },

  reducers: {
    saveTagList(state, action) {
      return {
        ...state,
        ...action.payload,
      }
    },
    changeLoading(state, action) {
      return {
        ...state,
        ...action.payload,
      }
    },
    saveItemData(state, action) {
      return {
        ...state,
        tagItem: {
          tagId: action.payload.tagId,
          tagName: action.payload.tagName,
          type: action.payload.type,
          merchantId: action.payload.merchantId,
          merchantName: action.payload.merchantName,
        }
      }
    },
    clearAll(state) {
      return {
        ...state,
        tagItem: TagItemFormTemp(),
      }
    }
  }
}
