import { notification, message } from 'antd';
import { getWareList, addWareItem, updateWareItem, delWareItem } from '../services/ProductApi';
import { dateFormat } from '../utils/utils';
import Logger from '../utils/logger';

const logger = new Logger('WareModel');

export default {
  namespace: 'ware',

  state: {
    data: {
      list: [],
      pagination: {},
    },
    loading: false,
    wareItem: {
      actionType: 'ADD',
      itemData: {},
    },
  },

  effects: {
    *getWareList({ payload }, { put, call }) {
      yield put({ type: 'changeLoading', payload: { loading: true } });
      const response = yield call(getWareList, payload);

      if (response.err)
      {
        logger.error('Ware list request error: %o', response.err);
        return;
      }
      else {
        logger.debug('Ware list request succ, response: %o', response);
      }

      const data = {
        list: dateFormat(response.data.list || []),
        pagination: {
          pageSize : response.data.pageSize || 15,
          current  : response.data.pageNo || 1,
          total    : response.data.totalSize || 0,
        }
      };
      logger.info('【Response Pagination】: %o', data);
      yield put({ type: 'saveWareList', payload: { data } });
      yield put({ type: 'changeLoading', payload: { loading: false }});
    },
    *addWareItem({ payload }, { put, call }) {
      const response = yield call(addWareItem, payload);
      if (!response || response.code !== 200) {
        notification.error({
          message: '新建课件失败!',
          description: response.message,
        });
      }
      else {
        notification.success({
          message: '新建课件成功!',
          description: '成功创建1个课件',
        });
      }
    },
    *updateWareItem({ payload }, { put, call }) {
      const response = yield call(updateWareItem, payload);
      if (!response || response.code !== 200) {
        notification.error({
          message: '更新课件失败!',
          description: response.message,
        });
      }
      else {
        notification.success({
          message: '更新课件成功!',
          description: '成功更新1个课件',
        });
      }
    },
    *delWareItem({ payload }, { put, call }) {
      const response = yield call(delWareItem, payload);
      if (!response || response.code !== 200) {
        notification.error({
          message: '删除课件失败!',
          description: response.message,
        });
      }
      else {
        notification.success({
          message: '删除课件成功!',
          description: '成功删除1个课件',
        });
      }
    }
  },

  reducers: {
    saveWareList(state, action) {
      return {
        ...state,
        ...action.payload,
      }
    },
    changeLoading(state, action) {
      return {
        ...state,
        ...action.payload,
      }
    },
    saveItemData(state, action) {
      return {
        ...state,
        wareItem: {
          itemData: action.payload,
          actionType: 'EDIT',
        }
      }
    },
    clearAll(state) {
      return {
        ...state,
        wareItem: {
          actionType: 'ADD',
          itemData: {},
        }
      }
    }
  },
}
