import { connect } from 'dva';
import { routerRedux } from 'dva/router';
import React, { PureComponent } from 'react';
import { Form, Input, Select, Card, Button, Table } from 'antd';

import PageHeaderLayout from '../../layouts/PageHeaderLayout';
import DescriptionList from '../../components/DescriptionList';
import EditableTable from '../../components/EditableTable';
import styles from './ItemSave.less';

import { getParentPath } from '../../utils/utils';
import config from '../../utils/config';
import Logger from '../../utils/logger';

const FormItem = Form.Item;
const InputGroup = Input.Group;
const { TextArea } = Input;
const { Option } = Select;
const { Description } = DescriptionList;

const { ROUTER_MAP, ACTION_MAP } = config;
const logger = Logger.getLogger('ItemSave');

@connect(state => ({
  item: state.item,
}))
@Form.create()
export default class ItemSave extends PureComponent {
  state = {
    endPoint: '',          //操作类型，edit
    parentPath: '',        //路径前缀
    currentTab: 'basic',   //当前选中的tab
  };

  _actionControlCenter = (type, payload = {}) => {
    const { dispatch } = this.props;
    //对空字段进行过滤
    Object.keys(payload).map(key => {payload[key] ? null : delete payload[key]});
    //触发modal中action
    dispatch({ type, payload });
  }

  _routerControlCenter = (router = '/', payload = null) => {
    const { dispatch } = this.props;
    if (payload) {
      dispatch(routerRedux.push({ pathname: router, state: payload }));
    }
    else {
      dispatch(routerRedux.push(router));
    }
  }

  componentWillMount() {
    const { location } = this.props;
    const { endPoint, parentPath } = getParentPath(location);
    if (endPoint === 'edit' && !location.state) {
      this._routerControlCenter(parentPath);
    }
    else {
      this.setState({ endPoint, parentPath });
    }
  }

  componentDidMount() {
    const { location } = this.props;

    this._actionControlCenter(ACTION_MAP.itemItem, location.state);
  }

  componentWillUnmount(){
    this._actionControlCenter(ACTION_MAP.itemClear);
    this.setState({ endPoint: '', parentPath: '' });
  }

  handlePageCancel = () => {
    routerControlCenter(ROUTER_MAP.itemList);
  }

  handlePageSubmit = (e) => {
    e.preventDefault();

    const { endPoint } = this.state;
    const { form, item, support: { supportItem } } = this.props;

    form.validateFields((err, fieldsValue) => {
      if (err) return;
      const values = { ...item.itemItem, ...fieldsValue};
      if (endPoint === 'edit')
      {
        actionControlCenter(ACTION_MAP.itemItem, values);
      }
      else if (endPoint === 'add')
      {
        actionControlCenter(ACTION_MAP.itemItemAdd, values);
      }
      else
      {
        message.error('未知操作类型!', 5); //不可能走到这里
      }
      form.resetFields();
      routerControlCenter(ROUTER_MAP.itemList);
    });
  }

  handleTabChange = (key) => {
    this.setState({ currentTab: key });
  }

  renderAboutList = (item) => {
    const lessonColumns = [{
      title: '课件编号',
      dataIndex: 'code',
    },{
      title: '课件名称',
      dataIndex: 'name',
    },{
      title: '课件封面',
      dataIndex: 'url',
    }];

    const courseColumns = [{
      title: '课编号',
      dataIndex: 'code',
    },{
      title: '课名称',
      dataIndex: 'name',
    },{
      title: '封面图片',
      dataIndex: 'url',
    }];

    const supportColumns = [{
      title: '周边编号',
      dataIndex: 'code',
    },{
      title: '周边名称',
      dataIndex: 'name',
    },{
      title: '封面图片',
      dataIndex: 'url',
    }];

    switch (item.type) {
      case 'LESSON':
        return (
          <Card title="课件列表" style={{ marginBottom: 24 }} bordered={true}>
            <Table columns={lessonColumns} bordered={true} dataSource={item.subList || []}/>
          </Card>
        );
      case 'COURSE':
        return (
          <Card title="课列表" style={{ marginBottom: 24 }} bordered={true}>
            <Table columns={courseColumns} bordered={true} dataSource={item.subList || []}/>
          </Card>
        );
      case 'SUPPORT':
        return (
          <Card title="周边列表" style={{ marginBottom: 24 }} bordered={true}>
            <Table columns={supportColumns} bordered={true} dataSource={item.subList || []}/>
          </Card>
        );
      default:
        break;
    }
  }

  render() {
    const { cp, tag, item: { itemItem } } = this.props;
    const { searchContent, currentTab } = this.state;
    const { getFieldDecorator } = this.props.form;

    const formItemLayout = {
      labelCol: {
        xs: { span: 24 },
        sm: { span: 7 },
      },
      wrapperCol: {
        xs: { span: 24 },
        sm: { span: 12 },
        md: { span: 12 },
      },
    };

    const submitFormLayout = {
      wrapperCol: {
        xs: { span: 24, offset: 0 },
        sm: { span: 10, offset: 7 },
      },
    };

    const tabList = [{
      key: 'basic',
      tab: '产品信息',
    },{
      key: 'price',
      tab: '产品定价',
    }];

    const aboutList = this.renderAboutList(itemItem);

    return(
      <PageHeaderLayout
      >
        <div className={styles.content}>
          <Card title="基础信息" style={{ marginBottom: 24 }} bordered={true}>
            <DescriptionList
              style={{ marginBottom: 24 }}
              col={1}
              size="large"
            >
              <Description term="产品编号">{itemItem.code}</Description>
              <Description term="产品名称">{itemItem.name}</Description>
              <Description term="产品类型"></Description>
              <Description term="产品标签"></Description>
              <Description term="产品概要">{itemItem.digest}</Description>
              <Description term="产品详情">{itemItem.digest}</Description>
              <Description term="封面图片"></Description>
            </DescriptionList>
          </Card>
          {aboutList}
          <Card title="产品价格" style={{ marginBottom: 24 }} bordered={true}>
            <EditableTable
              dataSource={itemItem.priceList}
              rowKeyName={"id"}
            />
          </Card>
          <Button>取消</Button>
          <Button style={{ marginLeft: 8 }} type="primary">提交</Button>
        </div>
      </PageHeaderLayout>
    );
  }
}
