import React, { PureComponent } from 'react';
import { Row, Col, Form, Button, Input, Select } from 'antd';
import { connect } from 'dva/router';

import PageHeaderLayout from '../../layouts/PageHeaderLayout';
import ModalTable from '../../components/ModalSelectTable';
import styles from './CourseSave.less';

const FormItem = Form.Item;

@Form.create()
export default class CourseSave extends PureComponent {
  state = {
    endPoint: '',
    parentPath: '',
    modalVisible: false,
  }

  componentWillMount() {
    const { dispatch, location } = this.props;
    let pathSnippets = location.pathname.split('/');
    const endPoint = pathSnippets.pop();
    const parentPath = pathSnippets.join('/') || '/';
    if (endPoint === 'edit' && !location.state) {
      dispatch(routerRedux.push(parentPath));
    }
    else {
      this.setState({ endPoint, parentPath });
    }
  }

  componentDidMount() {
    const { form, dispatch, location } = this.props;
    const { endPoint, parentPath } = this.state;

    if (endPoint !== 'add' && endPoint !== 'edit') {
      dispatch(routerRedux.push(parentPath));
      return;
    }

    if (endPoint === 'edit') {
      dispatch({
        type: 'lesson/getLessonOne',
        payload: location.state,
      });
    }
  }

  render() {
    const { getFieldDecorator } = this.props.form;

    return (
      <PageHeaderLayout>
        <div className={styles.content}>
          <Form onSubmit={this.handlePageSubmit}>
            <Row>
              <Col>
                <FormItem
                  label="课程编号"
                  {...formItemLayout}
                >
                  {getFieldDecorator('code', {
                      rules: [{ required: true, type: 'string', message: '课程编号必须填写!' }],
                      initialValue: lessonItem.code,
                  })(<Input placeholder="请输入课程编号" />)}
                </FormItem>
              </Col>
            </Row>
            <Row>
              <Col>
                <FormItem
                  label="课程名称"
                  {...formItemLayout}
                >
                  {getFieldDecorator('name', {
                    rules: [{ required: true, type: 'string', message: '课程名称必须填写!' }],
                    initialValue: lessonItem.name,
                  })(<Input placeholder="请输入课程名称" />)}
                </FormItem>
              </Col>
            </Row>
            <Row>
              <Col>
                <FormItem
                  label="课程描述"
                  {...formItemLayout}
                >
                  {getFieldDecorator('digest', {
                    initialValue: lessonItem.digest,
                  })(<TextArea rows={4} placeholder="请添加课程描述" />)}
                </FormItem>
              </Col>
            </Row>
            <Row>
              <Col>
                <FormItem
                  label="状态"
                  {...formItemLayout}
                >
                  {getFieldDecorator('status', {
                    initialValue: lessonItem.status,
                  })(
                    <Select>
                      {
                        config.LESSON_STATE.map(item =>
                          <Option key={item.value} value={item.value}>{item.name}</Option>
                        )
                      }
                    </Select>
                  )}
                </FormItem>
              </Col>
            </Row>
            <Row>
              <Col>
                <FormItem
                  label="选取课"
                  {...formItemLayout}
                >
                  <Button
                    icon="edit"
                    type="primary"
                    onClick={() => {this.handleEditWareBtnClick()}}
                  >
                    编辑课列表
                  </Button>
                </FormItem>
              </Col>
            </Row>
            <Row>
              <Col>
                <FormItem
                  {...submitFormLayout}
                >
                  <Table
                    columns={tableColumns}
                    dataSource={lessonItem.wareList}
                    scroll={{ y: 200 }}
                    pagination={{ pageSize: 50 }}
                  >
                  </Table>
                </FormItem>
              </Col>
            </Row>
            <Row>
              <Col>
                <FormItem
                  {...submitFormLayout}
                >
                  <Button onClick={this.handlePageCancel}>取消</Button>
                  <Button style={{ marginLeft: 8 }} type="primary" htmlType="submit">提交</Button>
                </FormItem>
              </Col>
            </Row>
          </Form>
        </div>
      </PageHeaderLayout>
    );
  }

}
