import { connect } from 'dva';
import { routerRedux } from 'dva/router';
import { cloneDeep } from 'lodash';
import React, { PureComponent } from 'react';
import { message, Radio, Popover, Table, Row, Col, Card, Icon, Button, Form, Input, Select, Spin, Modal } from 'antd';
import PageHeaderLayout from '../../layouts/PageHeaderLayout';
import styles from './LessonSave.less';

import config from '../../utils/config';
import Logger from '../../utils/logger';

const { Option } = Select;
const { TextArea } = Input;
const InputGroup = Input.Group;
const ButtonGroup = Button.Group;
const RadioGroup = Radio.Group;
const RadioButton = Radio.Button;
const FormItem = Form.Item;

const logger = Logger.getLogger('WareEditPage');

@connect(state => ({
  ware: state.ware,
  lesson: state.lesson,
}))
@Form.create()
export default class LessonSave extends PureComponent {
    state = {
      modalVisible: false,
      modalSearchContent: { key: 'name', value: '' },
      wareSelectedArr: [],
      tabSelected: 'WAIT',
      endPoint: '',
      parentPath: '',
    }

  componentWillMount() {
    const { dispatch, location } = this.props;
    let pathSnippets = location.pathname.split('/');
    const endPoint = pathSnippets.pop();
    const parentPath = pathSnippets.join('/') || '/';
    if (endPoint === 'edit' && !location.state) {
      dispatch(routerRedux.push(parentPath));
    }
    else {
      this.setState({ endPoint, parentPath });
    }
  }

  componentDidMount() {
    const { form, dispatch, location } = this.props;
    const { endPoint, parentPath } = this.state;

    if (endPoint !== 'add' && endPoint !== 'edit') {
      dispatch(routerRedux.push(parentPath));
      return;
    }

    if (endPoint === 'edit') {
      dispatch({
        type: 'lesson/getLessonOne',
        payload: location.state,
      });
    }
  }

  componentWillUnmount() {
    const { dispatch } = this.props;
    dispatch({
      type: 'lesson/clearAll',
    });
    this.setState({
      modalVisible: false,
      modalSearchContent: { key: 'name', value: '' },
      wareSelectedArr: [],
      tabSelected: 'WAIT',
    });
  }

  handlePageCancel = () => {
    const { dispatch } = this.props;
    dispatch(routerRedux.push('/product/lesson'));
  }

  handlePageSubmit = (e) => {
    e.preventDefault();

    const { endPoint } = this.state;
    const { form, dispatch, lesson: { lessonItem } } = this.props;

    form.validateFields((err) => {
      if (err) return;

      if (endPoint === 'edit')
      {
        dispatch({
          type: 'lesson/updateLessonItem',
          payload: lessonItem,
        });
      }
      else if (endPoint === 'add')
      {
        dispatch({
          type: 'lesson/addLessonItem',
          payload: lessonItem,
        });
      }
      else
      {
        message.error('未知操作类型!', 5);  //不可能走到这里
      }

      form.resetFields();
      dispatch(routerRedux.push('/product/lesson'));
    });
  }

  handleEditWareBtnClick = () => {
    this.handleModalVisible(true);
    this.props.dispatch({
      type: 'ware/getWareList',
      payload: {},
    });
  }

  handleModalVisible = (flag) => {
    this.setState({
      modalVisible: !!flag
    });
  }

  handleModalCancel = () => {
    this.setState({
      modalVisible: false,
      modalSearchContent: { key: 'name', value: '' },
      tabSelected: 'WAIT',
    });
  }

  handleModalOk = () => {
    this.setState({
      modalVisible: false,
      modalSearchContent: { key: 'name', value: '' },
      tabSelected: 'WAIT',
    });
  }

  handleModalSearchItemChange = (value) => {
    this.setState({
      modalSearchContent: { ...this.state.modalSearchContent, key: value },
    });
  }

  handleModalInputValueChange = (e) => {
    const value = e.target.value;
    this.setState({
      modalSearchContent: { ...this.state.modalSearchContent, value: value},
    });
  }

  handleModalSearchBtnClick = () => {
    const { dispatch } = this.props;

    const { modalSearchContent: { key, value } } = this.state;

    const payload = {};
    payload[key] = value;

    dispatch({
      type: 'ware/getWareList',
      payload: payload,
    });
  }

  handleModalResetBtnClick = () => {
    const { dispatch } = this.props;

    this.setState({
      modalSearchContent: { ...this.state.modalSearchContent, value: '' },
    });

    dispatch({
      type: 'ware/getWareList',
      payload: {},
    });
  }

  handleModalTableChange = (pagination) => {
    const { dispatch } = this.props;

    const { modalSearchContent: { key, value } } = this.state;

    const params = {
      pageNo: pagination.current,
      pageSize: pagination.pageSize,
    };

    params[key] = value;

    dispatch({
      type: 'ware/getWareList',
      payload: params,
    })
  }

  handleModalTableAdd = (record) => {
    const { dispatch, lesson: { lessonItem } } = this.props;
    let wareList = lessonItem.wareList || [];
    wareList.push(record);
    dispatch({
      type: 'lesson/saveItemData',
      payload: {...lessonItem, wareList},
    });
  }

  handleModalTabChange = (e) => {
    this.setState({
      tabSelected: e.target.value,
    });
  }

  handleModalSelectedItemDel = (record) => {
    const { dispatch, lesson: { lessonItem }} = this.props;
    const  beforeDel = lessonItem.wareList || [];
    const wareList = beforeDel.filter(item => item.id !== record.id);
    dispatch({
      type: 'lesson/saveItemData',
      payload: {...lessonItem, wareList},
    });
  }

  handleModalSort = (record, sort_up) => {
    const { dispatch, lesson: { lessonItem } } = this.props;

    const arr = cloneDeep(lessonItem.wareList);

    const index = arr.findIndex(item => item.id === record.id);

    if (sort_up)
    {
      //第一个元素或者未找到元素不做操作
      if (!index || -1 === index) return;
      //与前一个元素进行位置互换
      arr.splice(index, 1, ...arr.splice(index - 1, 1, arr[index]));
    }
    else
    {
      //最后一个元素或者未找到元素不做操作
      if (index + 1 === arr.length || -1 === index) return;
      //与后一个元素进行位置互换
      arr.splice(index, 1, ...arr.splice(index + 1, 1, arr[index]));
    }

    const wareList = arr;

    dispatch({
      type: 'lesson/saveItemData',
      payload: {...lessonItem, wareList},
    });
  }

  render() {
    const { modalVisible, modalSearchContent, tabSelected } = this.state;

    const { lesson: { lessonItem }, ware: { data: { list, pagination }, loading } } = this.props;

    const { getFieldDecorator } = this.props.form;

    const formItemLayout = {
      labelCol: {
        xs: { span: 24 },
        sm: { span: 7 },
      },
      wrapperCol: {
        xs: { span: 24 },
        sm: { span: 12 },
        md: { span: 10 },
      },
    };

    const submitFormLayout = {
      wrapperCol: {
        xs: { span: 24, offset: 0 },
        sm: { span: 10, offset: 7 },
      },
    };

    const tableColumns = [{
      title: '课件编号',
      dataIndex: 'code',
      width: '50%',
    },{
      title: '课件名称',
      dataIndex: 'name',
      width: '50%',
    }];

    const modalWaitSelectTableColumns = [{
      title: '课件编号',
      dataIndex: 'code',
      width: '40%',
    },{
      title: '课件名称',
      dataIndex: 'name',
      width: '45%'
    },{
      title: '操作',
      render: (text, record) => (
        <Button
          disabled={lessonItem.wareList.filter(item => item.id === record.id).length}
          size="small"
          type="dashed"
          icon="plus"
          onClick={() => {this.handleModalTableAdd(record)}}
        >
        </Button>
      ),
      width: '15%',
    }];

    const modalSelectedTableColumns = [{
      title: '课件编号',
      dataIndex: 'code',
      width: '30%',
    },{
      title: '课件名称',
      dataIndex: 'name',
      width: '35%',
    },{
      title: '排序',
      render: (text, record) => (
        <ButtonGroup>
          <Button size="small" icon="up" onClick={() => {this.handleModalSort(record, true)}}></Button>
          <Button size="small" icon="down" onClick={() => {this.handleModalSort(record, false)}}></Button>
        </ButtonGroup>
      ),
      width: '20%',
    },{
      title: '删除',
      render: (record) => (
        <Button size="small" icon="delete" onClick={() => this.handleModalSelectedItemDel(record)}></Button>
      ),
      width: '15%',
    }];

    return (
      <PageHeaderLayout>
        <div className={styles.content}>
          <Form onSubmit={this.handlePageSubmit}>
            <Row>
              <Col>
                <FormItem
                  label="课编号"
                  {...formItemLayout}
                >
                  {getFieldDecorator('code', {
                      rules: [{ required: true, type: 'string', message: '课编号必须填写!' }],
                      initialValue: lessonItem.code,
                  })(<Input placeholder="请输入课编号" />)}
                </FormItem>
              </Col>
            </Row>
            <Row>
              <Col>
                <FormItem
                  label="课名称"
                  {...formItemLayout}
                >
                  {getFieldDecorator('name', {
                    rules: [{ required: true, type: 'string', message: '课名称必须填写!' }],
                    initialValue: lessonItem.name,
                  })(<Input placeholder="请输入课名称" />)}
                </FormItem>
              </Col>
            </Row>
            <Row>
              <Col>
                <FormItem
                  label="课描述"
                  {...formItemLayout}
                >
                  {getFieldDecorator('digest', {
                    initialValue: lessonItem.digest,
                  })(<TextArea rows={4} placeholder="请添加课描述" />)}
                </FormItem>
              </Col>
            </Row>
            <Row>
              <Col>
                <FormItem
                  label="状态"
                  {...formItemLayout}
                >
                  {getFieldDecorator('status', {
                    initialValue: lessonItem.status,
                  })(
                    <Select>
                      {
                        config.LESSON_STATE.map(item =>
                          <Option key={item.value} value={item.value}>{item.name}</Option>
                        )
                      }
                    </Select>
                  )}
                </FormItem>
              </Col>
            </Row>
            <Row>
              <Col>
                <FormItem
                  label="选取课件"
                  {...formItemLayout}
                >
                  <Button
                    icon="edit"
                    type="primary"
                    onClick={() => {this.handleEditWareBtnClick()}}
                  >
                    编辑课件
                  </Button>
                </FormItem>
              </Col>
            </Row>
            <Row>
              <Col>
                <FormItem
                  {...submitFormLayout}
                >
                  <Table
                    columns={tableColumns}
                    dataSource={lessonItem.wareList}
                    scroll={{ y: 200 }}
                    pagination={{ pageSize: 50 }}
                  >
                  </Table>
                </FormItem>
              </Col>
            </Row>
            <Row>
              <Col>
                <FormItem
                  {...submitFormLayout}
                >
                  <Button onClick={this.handlePageCancel}>取消</Button>
                  <Button style={{ marginLeft: 8 }} type="primary" htmlType="submit">提交</Button>
                </FormItem>
              </Col>
            </Row>
          </Form>
        </div>
        {modalVisible ? (
          <Modal
            title="选择课件"
            width={600}
            maskClosable={false}
            visible={modalVisible}
            onOk={this.handleModalOk}
            onCancel={this.handleModalCancel}
          >
            <Row gutter={24}>
              <Col md={20}>
                <InputGroup compact>
                  <Select
                    defaultValue="name"
                    style={{ width: '25%' }}
                    onChange={this.handleModalSearchItemChange}
                  >
                    <Option value="name">课件名称</Option>
                    <Option value="code">课件编号</Option>
                  </Select>
                  <Input
                    placeholder="请输入搜索内容"
                    value={modalSearchContent.value}
                    onChange={this.handleModalInputValueChange}
                    onPressEnter={this.handleModalSearchBtnClick}
                    style={{ width: '75%' }}
                  />
                </InputGroup>
              </Col>
              <Col md={2}>
                <Button
                  type="primary"
                  icon="search"
                  shape="circle"
                  onClick={this.handleModalSearchBtnClick}
                >
                </Button>
              </Col>
              <Col md={2}>
                <Button
                  icon="reload"
                  shape="circle"
                  onClick={this.handleModalResetBtnClick}
                >
                </Button>
              </Col>
            </Row>
            <Row>
              <RadioGroup size="small" defaultValue="WAIT" onChange={this.handleModalTabChange} style={{ marginTop: 10, marginBottom: 10 }}>
                <RadioButton value="WAIT">待选</RadioButton>
                <RadioButton value="SELECTED">{`已选【${lessonItem.wareList.length}】`}</RadioButton>
              </RadioGroup>
            </Row>
            <Row>
              <Col>
                <Table
                  columns={modalWaitSelectTableColumns}
                  rowKey={record => record.id}
                  dataSource={list}
                  loading={loading}
                  onChange={this.handleModalTableChange}
                  pagination={pagination}
                  scroll={{ y: 200 }}
                  style={tabSelected === 'WAIT' ? null : { display: 'none' }}
                >
                </Table>
              </Col>
            </Row>
            <Row>
              <Col>
                <Table
                  style={tabSelected === 'SELECTED' ? null : { display: 'none' }}
                  columns={modalSelectedTableColumns}
                  rowKey={record => record.id}
                  dataSource={lessonItem.wareList}
                  pagination={{ pageSize: 50 }}
                  scroll={{ y: 200 }}
                >
                </Table>
              </Col>
            </Row>
          </Modal>
          ) : null
        }
      </PageHeaderLayout>
    )
  }
}
