import { connect } from 'dva';
import { routerRedux } from 'dva/router';
import React, { PureComponent } from 'react';
import { Form, Input, Select, Button, Table } from 'antd';

import PageHeaderLayout from '../../layouts/PageHeaderLayout';
import styles from './TagSave.less';
import config from '../../utils/config';
import Logger from '../../utils/logger';

const FormItem = Form.Item;
const InputGroup = Input.Group;
const { TextArea } = Input;
const { Option } = Select;

const logger = Logger.getLogger('TagSave');

@connect(state => ({
  tag: state.tag,
  merchant: state.merchant,
}))
@Form.create()
export default class TagSave extends PureComponent {
  state = {
    endPoint: '',          //操作类型，add或edit
    parentPath: '',        //路径前缀
    searchInput: '',       //模态框的搜索字段内容
  };

  _actionControlCenter(type, payload = {}) {
    const { dispatch } = this.props;
    //对空字段进行过滤
    Object.keys(payload).map(key => {payload[key] ? null : delete payload[key]});
    dispatch({ type, payload });
  }

  componentWillMount() {
    const { dispatch, location } = this.props;
    let pathSnippets = location.pathname.split('/');
    const endPoint = pathSnippets.pop();
    const parentPath = pathSnippets.join('/') || '/';
    if (endPoint === 'edit' && !location.state) {
      dispatch(routerRedux.push(parentPath));
    }
    else {
      this.setState({ endPoint, parentPath });
    }
  }

  componentDidMount() {
    const { form, dispatch, location } = this.props;
    const { endPoint, parentPath } = this.state;

    if (endPoint !== 'add' && endPoint !== 'edit') {
      dispatch(routerRedux.push(parentPath));
      return;
    }

    if (endPoint === 'edit') {
      this._actionControlCenter('tag/getTagOne', location.state);
    }

    this._actionControlCenter('merchant/getAllMerchants');
  }

  componentWillUnmount(){
    this._actionControlCenter('tag/clearAll');
    this.setState({
      endPoint: '',
      parentPath: '',
    });
  }

  handlePageCancel = () => {
    const { dispatch } = this.props;
    dispatch(routerRedux.push('/product/tag'));
  }

  handlePageSubmit = (e) => {
    e.preventDefault();

    const { endPoint } = this.state;
    const { form, dispatch, tag: { tagItem } } = this.props;

    form.validateFields((err, fieldsValue) => {
      if (err) return;

      const values = { ...tagItem, ...fieldsValue};

      if (endPoint === 'edit')
      {
        this._actionControlCenter('tag/updateTagItem', values);
      }
      else if (endPoint === 'add')
      {
        this._actionControlCenter('tag/addTagItem', values);
      }
      else
      {
        message.error('未知操作类型!', 5);  //不可能走到这里
      }

      form.resetFields();
      dispatch(routerRedux.push('/product/tag'));
    });
  }

  render() {
    const { tag: { tagItem }, merchant } = this.props;
    const { getFieldDecorator } = this.props.form;

    const formItemLayout = {
      labelCol: {
        xs: { span: 24 },
        sm: { span: 7 },
      },
      wrapperCol: {
        xs: { span: 24 },
        sm: { span: 12 },
        md: { span: 10 },
      },
    };

    const submitFormLayout = {
      wrapperCol: {
        xs: { span: 24, offset: 0 },
        sm: { span: 10, offset: 7 },
      },
    };

    return(
      <PageHeaderLayout>
        <div className={styles.content}>
          <Form onSubmit={this.handlePageSubmit}>
            <FormItem
              label="标签名称"
              {...formItemLayout}
            >
              {getFieldDecorator('tagName', {
                rules: [{ required: true, type: 'string', message: '标签名称必须填写!' }],
                initialValue: tagItem.tagName,
              })(<Input placeholder="请输入标签名称" />)}
            </FormItem>
            <FormItem
              label="标签类型"
              {...formItemLayout}
            >
              {getFieldDecorator('type', {
                  initialValue: tagItem.type,
              })(
                <Select
                  placeholder="请选择标签类型"
                >
                  {config.TAG_TYPE.map(item =>
                    <Option key={item.value} value={item.value}>{item.name}</Option>
                  )}
                </Select>
              )}
            </FormItem>
            <FormItem
              label="所属渠道方"
              {...formItemLayout}
            >
              {getFieldDecorator('merchantId', {
                  initialValue: tagItem.merchantId,
              })(
                <Select
                  placeholder="请选择所属渠道方"
                  notFoundContent="渠道方数据加载失败"
                >
                  {merchant.merchantList.map(item =>
                    <Option key={item.merchantId} value={item.merchantId}>{item.merchantName}</Option>
                  )}
                </Select>
              )}
            </FormItem>
            <FormItem
              {...submitFormLayout}
            >
              <Button onClick={this.handlePageCancel}>取消</Button>
              <Button style={{ marginLeft: 8 }} type="primary" htmlType="submit">提交</Button>
            </FormItem>
          </Form>
        </div>
      </PageHeaderLayout>
    );
  }
}
