import { connect } from 'dva';
import { routerRedux } from 'dva/router';
import { cloneDeep } from 'lodash';
import React, { PureComponent } from 'react';
import { message, Radio, Popover, Table, Row, Col, Card, Icon, Button, Form, Input, Select, Spin, Modal } from 'antd';
import PageHeaderLayout from '../../layouts/PageHeaderLayout';
import styles from './WareSave.less';

import config from '../../utils/config';
import Logger from '../../utils/logger';

const { Option } = Select;
const { TextArea } = Input;
const InputGroup = Input.Group;
const ButtonGroup = Button.Group;
const RadioGroup = Radio.Group;
const RadioButton = Radio.Button;
const FormItem = Form.Item;

const logger = Logger.getLogger('WareEditPage');

@connect(state => ({
  cp: state.cp,
  ware: state.ware,
  resource: state.resource,
}))
@Form.create()
export default class WareSave extends PureComponent {
  state = {
    wareType: config.WARE_TYPE_IMAGE, //课件的默认类型为图片类型
    videoSelected: [],
    modalVisible: false,
    modalSearchContent: { key: 'name', value: '' },
    imgSelectedArr: [],
    tabSelected: 'WAIT',
  }

  componentDidMount() {
    const { form, dispatch, ware: { wareItem } } = this.props;

    dispatch({
      type: 'cp/getCPList',
    });

    if (wareItem.actionType === 'EDIT')
    {
      form.setFieldsValue({
        name: wareItem.itemData.name,
        code: wareItem.itemData.code,
        digest: wareItem.itemData.digest,
        type: wareItem.itemData.type,
        cpId: wareItem.itemData.cpId,
      });
      this.setState({
        wareType: wareItem.itemData.type,
      });
    }
  }

  componentWillUnmount() {
    const { dispatch } = this.props;
    dispatch({
      type: 'ware/clearAll',
    });
    this.setState({
      wareType: config.WARE_TYPE_IMAGE,
      videoSelected: [],
      modalVisible: false,
      modalSearchContent: { key: 'name', value: '' },
      imgSelectedArr: [],
      tabSelected: 'WAIT',
    });
  }

  handlePageCancel = () => {
    const { dispatch } = this.props;
    dispatch(routerRedux.push('/product/ware'));
  }

  handlePageSubmit = (e) => {
    e.preventDefault();

    const { form, dispatch, ware: { wareItem } } = this.props;

    form.validateFields((err, fieldsValue) => {
      if (err) return;

      const values = { ...fieldsValue };

      values.imgIds = []; //TODO

      values.playUrl = ''; //TODO

      if ( wareItem.actionType === 'EDIT')
      {
        dispatch({
          type: 'ware/updateWareItem',
          payload: values,
        });
      }
      else if ( wareItem.actionType === 'ADD')
      {
        dispatch({
          type: 'ware/addWareItem',
          payload: values,
        });
      }
      else
      {
        message.error('未知操作类型!', 5000);  //不可能走到这里
      }

      form.resetFields();
      dispatch(routerRedux.push('/product/ware'));
    });
  }

  handleWareTypeChange = (value) => {
    this.setState({
      wareType: value,
    });
  }

  handleVideoSearch = (value) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'resource/fetch',
      payload: { type: config.RESOURCE_TYPE_IMAGE },
    });
  }

  handleVideoSelectChange = (value) => {
    this.setState({
      videoSelected: value,
    });
  }

  handleEditImageBtnClick = () => {
    this.handleModalVisible(true);
    this.props.dispatch({
      type: 'resource/fetch',
      payload: { type: config.RESOURCE_TYPE_IMAGE },
    });
  }

  handleModalVisible = (flag) => {
    this.setState({
      modalVisible: !!flag
    });
  }

  handleModalCancel = () => {
    this.setState({
      modalVisible: false,
      modalSearchContent: { key: 'name', value: '' },
      tabSelected: 'WAIT',
    });
  }

  handleModalOk = () => {
    this.setState({
      modalVisible: false,
      modalSearchContent: { key: 'name', value: '' },
      tabSelected: 'WAIT',
    });
  }

  handleModalSearchItemChange = (value) => {
    this.setState({
      modalSearchContent: { ...this.state.modalSearchContent, key: value },
    });
  }

  handleModalInputValueChange = (e) => {
    const value = e.target.value;
    this.setState({
      modalSearchContent: { ...this.state.modalSearchContent, value: value},
    });
  }

  handleModalSearchBtnClick = () => {
    const { dispatch } = this.props;
    const { modalSearchContent: { key, value } } = this.state;
    const payload = {};
    payload[key] = value;
    payload.type = config.RESOURCE_TYPE_IMAGE,

    dispatch({
      type: 'resource/fetch',
      payload: payload,
    });
  }

  handleModalResetBtnClick = () => {
    const { dispatch } = this.props;

    this.setState({
      modalSearchContent: { ...this.state.modalSearchContent, value: '' },
    });

    dispatch({
      type: 'resource/fetch',
      payload: { type: config.RESOURCE_TYPE_IMAGE },
    });
  }

  handleModalTableChange = (pagination) => {
    const { dispatch } = this.props;

    const { modalSearchContent: { key, value } } = this.state;

    const params = {
      pageNo: pagination.current,
      pageSize: pagination.pageSize,
      type: config.RESOURCE_TYPE_IMAGE,
    };

    params[key] = value;

    dispatch({
      type: 'resource/fetch',
      payload: params,
    })
  }

  handleModalTableAdd = (record) => {
    const tmp = cloneDeep(this.state.imgSelectedArr);
    tmp.push(cloneDeep(record));
    this.setState({
      imgSelectedArr: tmp,
    });
  }

  handleModalTabChange = (e) => {
    this.setState({
      tabSelected: e.target.value,
    });
  }

  handleModalSelectedItemDel = (record) => {
    this.setState({
      imgSelectedArr: this.state.imgSelectedArr.filter(item => item.key !== record.key),
    });
  }

  handleModalSort = (record, sort_up) => {
    const { imgSelectedArr } = this.state;

    const arr = cloneDeep(imgSelectedArr);

    const index = arr.findIndex(item => item.key === record.key);

    if (sort_up)
    {
      //第一个元素或者未找到元素不做操作
      if (!index || -1 === index) return;
      //与前一个元素进行位置互换
      arr.splice(index, 1, ...arr.splice(index - 1, 1, arr[index]));
    }
    else
    {
      //最后一个元素或者未找到元素不做操作
      if (index + 1 === arr.length || -1 === index) return;
      //与后一个元素进行位置互换
      arr.splice(index, 1, ...arr.splice(index + 1, 1, arr[index]));
    }

    this.setState({
      imgSelectedArr: arr,
    });
  }

  render() {
    const { wareType, videoSelected, modalVisible, modalSearchContent, imgSelectedArr, tabSelected } = this.state;

    const { cp: { cpList }, resource: { data: { list, pagination }, loading } } = this.props;

    const { getFieldDecorator } = this.props.form;

    const formItemLayout = {
      labelCol: {
        xs: { span: 24 },
        sm: { span: 7 },
      },
      wrapperCol: {
        xs: { span: 24 },
        sm: { span: 12 },
        md: { span: 10 },
      },
    };

    const submitFormLayout = {
      wrapperCol: {
        xs: { span: 24, offset: 0 },
        sm: { span: 10, offset: 7 },
      },
    };

    const tableColumns = [{
      title: '图片编号',
      dataIndex: 'code',
      width: 150,
    },{
      title: '图片名称',
      dataIndex: 'name',
      width: 150,
    },{
      title: '缩略图',
      dataIndex: 'url',
      render: (text, record) => (
        <Popover
          content={
            <img alt="" src={record.url} width={200} />
          }
          title={record.name}
        >
          <img alt="" src={record.url} width={70} />
        </Popover>
      ),
    }];

    const modalNotSelectTableColumns = [{
      title: '图片编号',
      dataIndex: 'code',
      width: 190,
    },{
      title: '图片名称',
      dataIndex: 'name',
      width: 190,
    },{
      title: '缩略图',
      dataIndex: 'url',
      render: (text, record) => (
        <Popover
          content={
            <img alt="" src={record.url} width={200} />
          }
          title={record.name}
        >
          <img alt="" src={record.url} width={50} />
        </Popover>
      ),
      width: 100,
    },{
      title: '添加',
      render: (text, record) => (
        <Button
          disabled={imgSelectedArr.filter(item => item.key === record.key).length}
          size="small"
          type="dashed"
          icon="plus"
          onClick={() => {this.handleModalTableAdd(record)}}
        >
        </Button>
      ),
    }];

    const modalSelectedTableColumns = [{
      title: '图片编号',
      dataIndex: 'code',
      width: 140,
    },{
      title: '图片名称',
      dataIndex: 'name',
      width: 140
    },{
      title: '缩略图',
      dataIndex: 'url',
      render: (text, record) => (
        <Popover
          content={
            <img alt="" src={record.url} width={200} />
          }
          title={record.name}
        >
          <img alt="" src={record.url} width={50} />
        </Popover>
      ),
      width: 100,
    },{
      title: '排序',
      render: (text, record) => (
        <ButtonGroup>
          <Button size="small" icon="up" onClick={() => {this.handleModalSort(record, true)}}></Button>
          <Button size="small" icon="down" onClick={() => {this.handleModalSort(record, false)}}></Button>
        </ButtonGroup>
      ),
      width: 90,
    },{
      title: '删除',
      render: (record) => (
        <Button size="small" icon="delete" onClick={() => this.handleModalSelectedItemDel(record)}></Button>
      ),
    }];

    return (
      <PageHeaderLayout>
        <div className={styles.content}>
          <Form onSubmit={this.handlePageSubmit}>
            <Row>
              <Col>
                <FormItem
                  label="课件编号"
                  {...formItemLayout}
                >
                  {getFieldDecorator('code', {
                    rules: [{ required: true, type: 'string', message: '课件编号必须填写!' }]
                  })(<Input placeholder="请输入课件编号" />)}
                </FormItem>
              </Col>
            </Row>
            <Row>
              <Col>
                <FormItem
                  label="课件名称"
                  {...formItemLayout}
                >
                  {getFieldDecorator('name', {
                    rules: [{ required: true, type: 'string', message: '课件名称必须填写!' }]
                  })(<Input placeholder="请输入课件名称" />)}
                </FormItem>
              </Col>
            </Row>
            <Row>
              <Col>
                <FormItem
                  label="课件描述"
                  {...formItemLayout}
                >
                  {getFieldDecorator('digest')(
                    <TextArea rows={4} placeholder="请添加课件描述" />
                  )}
                </FormItem>
              </Col>
            </Row>
            <Row>
              <Col>
                <FormItem
                  label="课件类型"
                  {...formItemLayout}
                >
                  {getFieldDecorator('type', {
                    rules: [{ required: true, type: 'number', message: '课件类型必须选择!' }]
                  })(
                    <Select placeholder="请选择课件类型" onChange={this.handleWareTypeChange}>
                      {
                        config.WARE_TYPE.map(item =>
                          <Option key={item.value} value={item.value}>{item.name}</Option>
                        )
                      }
                    </Select>
                  )}
                </FormItem>
              </Col>
            </Row>
            <Row>
              <Col>
                <FormItem
                  label="内容供应商"
                  {...formItemLayout}
                >
                  {getFieldDecorator('cpId')(
                    <Select placeholder="请选择内容提供商">
                      {
                        cpList.map(item =>
                          <Option key={item.cpId} value={item.cpId}>{item.cpName}</Option>
                        )
                      }
                    </Select>
                  )}
                </FormItem>
              </Col>
            </Row>
            {wareType === config.WARE_TYPE_VIDEO ? (
                <Row>
                  <Col>
                    <FormItem
                      label="选择视频"
                      {...formItemLayout}
                    >
                      <Select
                        mode="multiple"
                        value={videoSelected}
                        placeholder="输入视频编号/名称进行搜索"
                        notFoundContent={loading ? <Spin size="small" /> : null}
                        filterOptions={false}
                        onSearch={this.handleVideoSearch}
                        onChange={this.handleVideoSelectChange}
                        style={{ width: '100%' }}
                      >
                        {
                          list.map(item =>
                            <Option
                              key={item.key}
                              value={item.key}
                            >{`${item.name}/${item.code}`}</Option>
                          )
                        }
                      </Select>
                    </FormItem>
                  </Col>
                </Row>
              ) : null
            }
            <Row>
              <Col>
                <FormItem
                  label="选取图片"
                  {...formItemLayout}
                >
                  <Button
                    icon="edit"
                    type="primary"
                    onClick={() => {this.handleEditImageBtnClick()}}
                    disabled={wareType === config.WARE_TYPE_VIDEO ? true: false}
                  >
                    编辑图片
                  </Button>
                </FormItem>
              </Col>
            </Row>
            <Row>
              <Col>
                <FormItem
                  {...submitFormLayout}
                >
                  <Table
                    columns={tableColumns}
                    dataSource={[]}
                    scroll={{ y: 200 }}
                    pagination={{ pageSize: 50 }}
                  >
                  </Table>
                </FormItem>
              </Col>
            </Row>
            <Row>
              <Col>
                <FormItem
                  {...submitFormLayout}
                >
                  <Button onClick={this.handlePageCancel}>取消</Button>
                  <Button style={{ marginLeft: 8 }} type="primary" htmlType="submit">提交</Button>
                </FormItem>
              </Col>
            </Row>
          </Form>
        </div>
        {modalVisible ? (
          <Modal
            title="组合图片"
            width={600}
            maskClosable={false}
            visible={modalVisible}
            onOk={this.handleModalOk}
            onCancel={this.handleModalCancel}
          >
            <Row gutter={24}>
              <Col md={20}>
                <InputGroup compact>
                  <Select
                    defaultValue="name"
                    style={{ width: '30%' }}
                    onChange={this.handleModalSearchItemChange}
                  >
                    <Option value="name">图片名称</Option>
                    <Option value="code">图片编号</Option>
                  </Select>
                  <Input
                    placeholder="请输入搜索内容"
                    value={modalSearchContent.value}
                    onChange={this.handleModalInputValueChange}
                    onPressEnter={this.handleModalSearchBtnClick}
                    style={{ width: '70%' }}
                  />
                </InputGroup>
              </Col>
              <Col md={2}>
                <Button
                  type="primary"
                  icon="search"
                  shape="circle"
                  onClick={this.handleModalSearchBtnClick}
                >
                </Button>
              </Col>
              <Col md={2}>
                <Button
                  icon="reload"
                  shape="circle"
                  onClick={this.handleModalResetBtnClick}
                >
                </Button>
              </Col>
            </Row>
            <Row>
              <RadioGroup size="small" defaultValue="WAIT" onChange={this.handleModalTabChange} style={{ marginTop: 10, marginBottom: 10 }}>
                <RadioButton value="WAIT">待选</RadioButton>
                <RadioButton value="SELECTED">{`已选【${imgSelectedArr.length}】`}</RadioButton>
              </RadioGroup>
            </Row>
            <Row>
              <Col>
                <Table
                  columns={modalNotSelectTableColumns}
                  rowKey={record => record.key}
                  dataSource={list}
                  loading={loading}
                  onChange={this.handleModalTableChange}
                  pagination={pagination}
                  scroll={{ y: 200 }}
                  style={tabSelected === 'WAIT' ? null : { display: 'none' }}
                >
                </Table>
              </Col>
            </Row>
            <Row>
              <Col>
                <Table
                  style={tabSelected === 'SELECTED' ? null : { display: 'none' }}
                  columns={modalSelectedTableColumns}
                  rowKey={record => record.key}
                  dataSource={imgSelectedArr}
                  pagination={{ pageSize: 50 }}
                  scroll={{ y: 200 }}
                >
                </Table>
              </Col>
            </Row>
          </Modal>
          ) : null
        }
      </PageHeaderLayout>
    )
  }
}
