import React, { PureComponent } from 'react';
import { connect } from 'dva';
import moment from 'moment';
import { Progress, Notification, Popover, Badge, message, Upload, Card, Icon, Button, Modal, Input, Select, Form, Row, Col, DatePicker, Tooltip } from 'antd';
import PageHeaderLayout from '../../layouts/PageHeaderLayout';
import StandardTable from '../../components/StandardTable';
import config from '../../utils/config';
import styles from './ImageList.less';

const { Option, OptGroup } = Select;
const FormItem = Form.Item;
const ButtonGroup = Button.Group;
const getValue = obj => Object.keys(obj).map(key => obj[key]).join(',');

@connect(state => ({
  resource: state.resource
}))
@Form.create()
export default class ImageList extends PureComponent {
    state = {
      selectedRows: [],
      modalVisible: false,
      formValues: {},
      selectSearchItem: 'name',
    };

    componentDidMount() {
      const { dispatch } = this.props;
      dispatch({
        type: 'resource/fetch',
        payload: { type: config.RESOURCE_TYPE_IMAGE },
      })
    }

    //响应不同搜索字段的变化
    handleSelectChange = (value) => {
      this.setState({
        selectSearchItem: value,
      });
    }

    //点击搜索按钮，提交表单，加载匹配内容
    handleSearchBtnClick = (e) => {
      e.preventDefault();

      const { dispatch, form } = this.props;

      form.validateFields((err, fieldsValue) => {
        if (err) return;

        if (fieldsValue['gmtCreated']) {
          fieldsValue['gmtCreated'] = fieldsValue['gmtCreated'].format('YYYY-MM-DD');
        }

        if (fieldsValue['gmtModified']) {
          fieldsValue['gmtModified'] = fieldsValue['gmtModified'].format('YYYY-MM-DD');
        }

        const values = {
          ...fieldsValue,
        };

        // dispatch({
        //   type: 'resource/fetch',
        //   payload: { type: config.RESOURCE_IMAGE_TYPE },
        // });
      });
    }

    //点击重置按钮处理,重置搜索表单，加载第一页数据
    handleReset = () => {
      this.props.form.resetFields();
      this.props.resource.data.list = []; //清空本地图片存储
      this.props.dispatch({
        type: 'resource/fetch',
        payload: { type: config.RESOURCE_IMAGE_TYPE },
      });
    }

    //响应行选择事件
    handleSelectRows = (rows) => {
      this.setState({
        selectedRows: rows,
      });
    }

    //配置分页器
    handleStandardTableChange = (pagination, filterArgs, sorter) => {
      const { dispatch } = this.props;
      const { formValues } = this.state;

      const filters = Object.keys(filterArgs).reduce((obj, key) => {
        const newObj = {...obj};
        newObj[key] = getValue(filterArgs[key]);
        return newObj;
      }, {});

      const params = {
        currentPage: pagination.pageNo,
        pageSize: pagination.pageSize,
        ...formValues,
        ...filters,
      };
      if (sorter.field) {
        params.sorter = `${sorter.field}_${sorter.order}`;
      }

      dispatch({
        type: 'resource/fetch',
        payload: params,
      })
    }

    //图片上传前处理
    handleBeforeUpload = (file) => {
      const { dispatch, resource: { oss: { fileList } } } = this.props;
      dispatch({
        type: 'resource/setFileList',
        payload: {
          fileList: [...fileList, file],
        }
      });
      return false;   //选则完文件后先不上传，等待点击确定按钮
    }

    //删除已选图片
    handleRemove = (file) => {
      const { dispatch, resource: { fileList } } = this.props;
      const index = fileList.indexOf(file);
      const newFileList = fileList.slice();
      newFileList.splice(index, 1);
      dispatch({
        type: 'resource/setFileList',
        payload: {
          fileList: newFileList,
        }
      })
    }

    //点击确定，进行一些列上传动作
    handleUploadSubmit = (e) => {
      const { dispatch } = this.props;

      // dispatch({
      //   type: 'resource/upload',
      // });

      this.setState({ modalVisible: false });

      Notification.success({
        message: '图片上传成功',
        description: '100%',
      });
      // e.preventDefault();
      // const { dispatch, form } = this.props;
      // form.validateFields(({fieldName: ['newName', 'newCode']}, err, fieldsValue) => {
      //   if (err) return;
      //   const values = {
      //     ...fieldsValue,
      //   };
      //   dispatch({
      //     type: 'resource/resourceUpload',
      //   });
      // });
    }

    //控制模态框显现状态
    handleModalVisible = (flag) => {
      this.setState({
        modalVisible: !!flag,
      });
    }

    //模态框表单提交，上传图片信息到应用服务器
    handleModalSubmit = (e) => {
      e.preventDefault();

      const { dispatch, form } = this.props;

      form.validateFields((err, fieldsValue) => {
        if (err) return;
      });
    }

    dynamicLoadWidget = () => {
      const { getFieldDecorator } = this.props.form;

      let config = {};

      switch (this.state.selectSearchItem) {
        case 'name':
          config = {
            rules: [{
              type: 'string',
              required: true,
              message: '请填入图片名称!',
            }],
          }
          return (
            <FormItem>
              {getFieldDecorator('name', config)(
                <Input placeholder="请输入图片名称" style={{ minWidth: 200, maxWidth: 250, width: '100%' }}></Input>
              )}
            </FormItem>
          );
        case 'code':
          config = {
            rules: [{
              type: 'string',
              required: true,
              message: '请填入图片编号!',
            }],
          }
          return (
            <FormItem>
              {getFieldDecorator('code', config)(
                <Input placeholder="请输入图片编号" style={{ minWidth: 200, maxWidth: 250, width: '100%' }}></Input>
              )}
            </FormItem>
          );
        case 'gmtCreated':
          config = {
            rules: [{
              type: 'object',
              required: true,
              message: '请选择创建日期!',
            }],
          }
          return (
            <FormItem>
              {getFieldDecorator('gmtCreated', config)(
                <DatePicker style={{ minWidth: 200, maxWidth: 250, width: '100%' }} />
              )}
            </FormItem>
          );
        case 'gmtModified':
          config = {
            rules: [{
              type: 'object',
              required: true,
              message: '请选择修改日期!',
            }],
          }
          return (
            <FormItem>
              {getFieldDecorator('gmtModified', config)(
                <DatePicker style={{ minWidth: 200, maxWidth: 250, width: '100%' }} />
              )}
            </FormItem>
          );
      }
    }

    render() {
      const { modalVisible, selectedRows } = this.state;

      const { getFieldDecorator } = this.props.form;

      const { dispatch, resource } = this.props;

      /* 列表表头定义 */
      const columns = [
        {
          title: '图片编码',
          dataIndex: 'code',
        },
        {
          title: '图片名称',
          dataIndex: 'name',
        },
        {
          title: '图片大小',
          dataIndex: 'size',
        },
        {
          title: '修改时间',
          dataIndex: 'gmtModified',
          sorter: true,
          render: val => <span>{moment(val).format('YYYY-MM-DD HH:mm:ss')}</span>,
        },
        {
          title: '缩略图',
          dataIndex: 'url',
          render: (text, record) => (
            <Popover
              content={
                <img alt="" src={record.url} width={350} />
              }
              title={record.name}
            >
              <img alt="" src={record.url} width={70} />
            </Popover>
          ),
        },
        {
          title: '操作',
          render: () => (
            <p>
              <Icon type="edit" />
              <span className={styles.splitLine} />
              <Icon type="delete" />
            </p>
          ),
        },
      ];

      const standardTableProps = {
        dataSource: resource.data.list,
        pagination: resource.data.pagination,
        loading: resource.loading,
        columns: columns,
        rowKeyName: 'code',
        selectedRows: selectedRows,
      };

      /* 顶部的筛选组件 */
      const headerSearchForm = (
        <Form layout="inline" onSubmit={this.handleSearchBtnClick}>
          <FormItem>
            <Select style={{ width: 150 }} defaultValue="name" onChange={this.handleSelectChange}>
              <Option value="name">图片名称</Option>
              <Option value="code">图片编号</Option>
              <Option value="gmtCreated">创建时间</Option>
              <Option value="gmtModified">修改时间</Option>
            </Select>
          </FormItem>
          {this.dynamicLoadWidget()}
          <FormItem>
            <ButtonGroup>
              <Button type="primary" icon="search" htmlType="submit">搜索</Button>
              <Button type="danger" icon="reload" onClick={this.handleReset}>重置</Button>
            </ButtonGroup>
          </FormItem>
        </Form>
      );

      /* 模态弹框 */
      const modalForm = (
        <Modal
          title="新建图片"
          visible={modalVisible}
          onOk={this.handleUploadSubmit}
          onCancel={() => this.handleModalVisible(false)}
        >
          <FormItem
            label="图片名称"
            labelCol={{ span: 5 }}
            wrapperCol={{ span: 15 }}
          >
            {getFieldDecorator('name', { rules: [{type:'string', required:true, message:'请填写图片名!'}] })(
              <Input placeholder="请输入图片名称" />
            )}
          </FormItem>
          <FormItem
            label="图片编号"
            labelCol={{ span: 5 }}
            wrapperCol={{ span: 15 }}
          >
            {getFieldDecorator('code')(
              <Input placeholder="请输入图片编号" />
            )}
          </FormItem>
          <FormItem
            label="选择图片"
            labelCol={{ span: 5 }}
            wrapperCol={{ span: 15 }}
          >
            <Upload
              beforeUpload={this.handleBeforeUpload}
              onRemove={this.handleRemove}
            >
              <Button icon="cloud-upload">选择图片</Button>
            </Upload>
          </FormItem>
        </Modal>
      );

      return (
        <PageHeaderLayout
        >
          <div className={styles.content}>
            <div className={styles.search}>{headerSearchForm}</div>
            <div className={styles.tableList}>
              <div className={styles.tableListOperator}>
                <Button type="primary" icon="plus" onClick={() => {this.handleModalVisible(true)}}>增加图片</Button>
              </div>
              <StandardTable
                {...standardTableProps}
                onSelectRow={this.handleSelectRows}
                onChange={this.handleStandardTableChange}
              />
            </div>
            <div>{modalForm}</div>
          </div>
      </PageHeaderLayout>
    );
  }
}
