import React, { PureComponent } from 'react';
import { connect } from 'dva';
import { Progress, notification, Badge, message, Card, Icon, Button,
  Modal, Input, Select, Form, Row, Col, Tooltip } from 'antd';
import moment from 'moment';

import PageHeaderLayout from '../../layouts/PageHeaderLayout';
import StandardTable from '../../components/StandardTable';
import config from '../../utils/config';

const { Option, OptGroup } = Select;

const FormItem = Form.Item;

const ButtonGroup = Button.Group;

const getValue = obj => Object.keys(obj).map(key => obj[key]).join(',');

@connect(state => ({
  tag: state.tag,
}))
@Form.create()
export default class VideoList extends PureComponent {
  state = {
    selectedRows: [],
    formValues: {},
  }

  componentDidMount() {
    const { dispatch } = this.props;
    dispatch({
      type: 'tag/getTagList',
      payload: {},
    })
  }

  handleSearch = (e) => {
    e.preventDefault();
    const { dispatch, form } = this.props;
    form.validateFields((err, fieldsValue) => {
      if (err) return;
      const values = {
        ...fieldsValue,
      }
      logger.info('【Search Values】: %o', values);
      dispatch({
        type: 'tag/getTagList',
        payload: values,
      });
    });
  }

  handleReset = (e) => {
    e.preventDefault();
    const { dispatch, form } = this.props;
    form.resetFields();
    dispatch({
      type: 'tag/getTagList',
      payload: {},
    });
  }

  handleSelectRows = (rows) => {
    this.setState({
      selectedRows: rows,
    });
  }

  //配置分页器
  handleStandardTableChange = (pagination, filterArgs, sorter) => {
    const { dispatch } = this.props;
    const { formValues } = this.state;

    logger.info('【TableChangePagination】: %o', pagination);
    const filters = Object.keys(filterArgs).reduce((obj, key) => {
      const newObj = {...obj};
      newObj[key] = getValue(filterArgs[key]);
      return newObj;
    }, {});

    const params = {
      pageNo: pagination.current,
      pageSize: pagination.pageSize,
      ...formValues,
      ...filters,
    };
    if (sorter.field) {
      params.sorter = `${sorter.field}_${sorter.order}`;
    }

    dispatch({
      type: 'tag/getTagList',
      payload: params,
    })
  }

  render() {
    const { selectedRows } = this.state;

    const { dispatch } = this.props;

    const { getFieldDecorator } = this.props.form;

    const pageHeaderSearch = (
      <Form layout="inline" onSubmit={this.handleSearch}>
        <Row gutter={24}>
          <Col md={9} sm={24}>
            <FormItem label="视频编号">
              {getFieldDecorator('videoName')(
                <Input placeholder="请输入视频编号" />
              )}
            </FormItem>
          </Col>
          <Col md={10} sm={24}>
            <FormItem label="视频名称">
              {getFieldDecorator('videoName')(
                <Input placeholder="请输入视频名称" />
              )}
            </FormItem>
          </Col>
          <Col md={5} sm={24}>
            <FormItem>
              <ButtonGroup>
                <Button icon="search" type="primary" htmlType="submit">搜索</Button>
                <Button icon="reload" type="danger" onClick={this.handleReset}>重置</Button>
              </ButtonGroup>
            </FormItem>
          </Col>
        </Row>
      </Form>
    );

    return (
      <PageHeaderLayout
        content={pageHeaderSearch}
      >
      </PageHeaderLayout>
    );
  }
}
