import { stringify } from 'qs';
import { cloneDeep } from 'lodash';
import { message } from 'antd';
import request from '../utils/request';
import config from '../utils/config';

const { api } = config;
const {
  tagList, tagItem, tagItemAdd, tagItemUpdate, tagItemDel,
  wareList, wareItemAdd, wareItemUpdate, wareItemDel,
  lessonList, lessonItem, lessonItemAdd, lessonItemUpdate, lessonItemDel,
  courseList, courseItem, courseItemAdd, courseItemUpdate, courseItemDel,
  supportList, supportItem, supportItemAdd, supportItemUpdate, supportItemDel,
} = api;

export async function getWareList(params) {
  return request(`${wareList}?${stringify(params)}`);
}

export async function addWareItem(params) {
  const options = {
    method: 'POST',
    body: { ...params },
  };
  return request(`${wareItemAdd}`, options);
}

export async function updateWareItem(params) {
  const options = {
    method: 'PUT',
    body: { ...params },
  };
  return request(`${wareItemUpdate}`, options);
}

export async function delWareItem(params) {
  const options = {
    method: 'DELETE',
    body: { ...params },
  };
  return request(`${wareItemDel}`, options);
}

export async function getLessonList(params) {
  return request(`${lessonList}?${stringify(params)}`);
}

export async function getLessonItem(params) {
  let apiPath = lessonItem.split('/:');
  const queryBy = apiPath.pop();
  const code = params[queryBy];
  return request(`${apiPath.join('/')}/${code}`);
}

export async function addLessonItem(params) {
  const options = {
    method: 'POST',
    body: { ...params },
  };
  return request(`${lessonItemAdd}`, options);
}

export async function updateLessonItem(params) {
  const options = {
    method: 'PUT',
    body: { ...params },
  };
  return request(`${lessonItemUpdate}`, options);
}

export async function delLessonItem(params) {
  const options = {
    method: 'DELETE',
    body: { ...params },
  };
  return request(`${lessonItemDel}`, options);
}

export async function getCourseList(params) {
  return request(`${courseList}?${stringify(params)}`);
}

export async function getCourseItem(params) {
  let apiPath = courseItem.split('/:');
  const queryBy = apiPath.pop();
  const code = params[queryBy];
  return request(`${apiPath.join('/')}/${code}`);
}


export async function addCourseItem(params) {
  const options = {
    method: 'POST',
    body: { ...params },
  };
  return request(`${courseItemAdd}`, options);
}

export async function updateCourseItem(params) {
  const options = {
    method: 'PUT',
    body: { ...params },
  };
  return request(`${courseItemUpdate}`, options);
}

export async function delCourseItem(params) {
  const options = {
    method: 'DELETE',
    body: { ...params },
  };
  return request(`${courseItemDel}`, options);
}

export async function getSupportList(params) {
  return request(`${supportList}?${stringify(params)}`);
}

export async function getSupportItem(params) {
  let apiPath = supportItem.split('/:');
  const queryBy = apiPath.pop();
  const code = params[queryBy];
  return request(`${apiPath.join('/')}/${code}`);
}

export async function addSupportItem(params) {
  const options = {
    method: 'POST',
    body: { ...params },
  };
  return request(`${supportItemAdd}`, options);
}

export async function updateSupportItem(params) {
  const options = {
    method: 'PUT',
    body: { ...params },
  };
  return request(`${supportItemUpdate}`, options);
}

export async function delSupportItem(params) {
  const options = {
    method: 'DELETE',
    body: { ...params },
  };
  return request(`${supportItemDel}`, options);
}

export async function getTagList(params) {
  return request(`${tagList}?${stringify(params)}`);
}

export async function getTagItem(params) {
  let apiPath = tagItem.split('/:');
  const queryBy = apiPath.pop();
  const id = params[queryBy];
  return request(`${apiPath.join('/')}/${id}`);
}

export async function addTagItem(params) {
  const options = {
    method: 'POST',
    body: { ...params },
  };
  return request(`${tagItemAdd}`, options);
}

export async function updateTagItem(params) {
  const options = {
    method: 'PUT',
    body: { ...params },
  };
  return request(`${tagItemUpdate}`, options);
}

export async function delTagItem(params) {
  const options = {
    method: 'DELETE',
    body: { ...params },
  };
  return request(`${tagItemDel}`, options);
}
