import navData from '../common/ljNav';
import cloneDeep from 'lodash/cloneDeep';

function timestampToStr(timestamp) {
  const year  = timestamp.getFullYear();
  const month = timestamp.getMonth() + 1;
  const day   = timestamp.getDate();
  const hour  = timestamp.getHours() === 0 ? '00' : timestamp.getHours();
  const minute= timestamp.getMinutes();
  const second= timestamp.getSeconds();
  return `${year}-${month}-${day} ${hour}:${minute}:${second}`;
}

export function dateFormat(dataList) {
  let formatedDataList = [];
  for (let obj of dataList) {
    if ('object' === typeof(obj)) {
      if (obj.hasOwnProperty('gmtModified') && 'number' === typeof(obj.gmtModified)) {
        obj.gmtModified = timestampToStr(new Date(obj.gmtModified));
        formatedDataList.push(obj);
      }
      else if (obj.hasOwnProperty('gmtCreated') && 'number' === typeof(obj.gmtCreated)) {
        obj.gmtCreated = timestampToStr(new Date(obj.gmtCreated));
        formatedDataList.push(obj);
      }
      else {
        formatedDataList.push(obj);
      }
    }
    else {
      formatedDataList.push(obj);
    }
  }
  return formatedDataList;
}

function getPlainNode(nodeList, parentPath = '') {
  const arr = [];
  nodeList.forEach((node) => {
    const item = node;
    item.path = `${parentPath}/${item.path || ''}`.replace(/\/+/g, '/');
    item.exact = true;
    if (item.children && !item.component) {
      arr.push(...getPlainNode(item.children, item.path));
    } else {
      if (item.children && item.component) {
        item.exact = false;
        arr.push(...getPlainNode(item.children, item.path));
      }
      arr.push(item);
    }
  });
  return arr;
}

export function getRouteData(path) {
  if (!navData.some(item => item.layout === path) ||
      !(navData.filter(item => item.layout === path)[0].children)) {
    return null;
  }
  const dataList = cloneDeep(navData.filter(item => item.layout === path)[0]);
  const nodeList = getPlainNode(dataList.children);
  return nodeList;
}

export const getParentPath = (location) => {
  let pathSnippets = location.pathname.split('/');
  const endPoint = pathSnippets.pop();
  const parentPath = pathSnippets.join('/') || '/';
  return { endPoint, parentPath };
}
